#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <fcntl.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "pfunc.h"
#include "sfunc.h"
#include "mfunc.h"
#include "telefono.h"
#include "iofile.h"
#include "rafobj.h"

#ifndef ANJUTA_MAKEFILE
#define SUB_CALLING
#include "calls01.c"	// Modifica Card
#include "calls02.c"	// Creazione Modello
#include "calls03.c"	// Gestione File
#include "calls04.c"	// Gestione Esportazione
#include "pfunc.c"
#include "sfunc.c"
#include "mfunc.c"

#include "telefono.c"
#include "iofile.c"
#include "rafobj.c"
#endif


/* Tutte le chiamate alle funzioni generate da Glade dovrebbero
 essere sostituite con delle chiamate a delle funzioni definite
 dal programmatore per evitare che l'eliminazione di un segnale
 nel progetto di Glade faccia sparire il codice della funzione
 associata a quel segnale. Ad ogni modo Glade non elimina codice */


void on_nuova_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	int i;
	FILE *fp;

	if (Status[S_RUBRICA]) {
		if (!access("NewRubrica.rbc", R_OK))
#ifdef ITALIANO
			show_error_msg("ERRORE: il file NewRubrica.rbc esiste gi: devi rinominalo.");
#else
			show_error_msg("ERROR: the file NewRubrica.rbc exist: you have to change name.");
#endif
		else {
			fp = fopen("NewRubrica.rbc", "w");
			if (fp == NULL)
#ifdef ITALIANO
				g_print("\nERRORE: non posso creare il file NewRubrica.rbc");
#else
				g_print("\nERROR: I cannot create file NewRubrica.rbc");
#endif
			else {
				for (i = 0; i < Status[S_CAMPI]; i++)
					fprintf(fp, "\"%s\"\t", Campi[i]);
				fprintf(fp, "\n\"#001\"\t");
				for (i = 1; i < Status[S_CAMPI]; i++)
					fprintf(fp, "\"_\"\t");
				fprintf(fp, "\n%s", Sequenza);
				fclose(fp);
			}
		}
	} else
#ifdef ITALIANO
		show_error_msg("ERRORE: nessuna rubrica caricata ! Per fare un modello vuoto ho bisogno di ispirarmi ad una rubrica aperta.");
#else
		show_error_msg("ERROR: first of all you need load a file ! I need a loaded example to do an empty file.");
#endif
}


void on_esci_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	if (Status[S_MODIFICA]) {
		/* Se esiste uno stato DESTROY lo preserva */
		if (Status[S_AVVRTM] & AVT_DESTROY)
			chiudi_con_modifiche(Base, AVT_BASE | AVT_DESTROY);
		else
			chiudi_con_modifiche(Base, AVT_BASE);
	} else if (Status[S_INIBIT] != S_RUBRICA) {
		save_init_values();
		g_print("\n");
		gtk_exit(0);
	}
}


void on_aggiungi_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	int new, cod, lst;

	if (Status[S_RUBRICA]) {
		get_new_code(&cod, &lst);
		if (cod > Status[S_CARDS]) {
			if (cod < 1000) {
				g_print("\nNew card (%d) initializing...", cod);

				new = create_n_link_new_card(cod, lst);
				Status[S_SELISTA] = new;
				// imposta_lista ();
				stampa_card(Lista[Status[S_SELISTA]], Base, "text3");
				on_modifica_activate(NULL, NULL);

				/* L'elemento selezionato diventa quello appena creato ma
				 * capita che esca della finestra di visualizzazione */
				// OGGI set_lista_item(new, !S_INIBIT); 

				Status[S_MODIFICA] = 1;
				set_sensitive_tools(IF_NEED);
			} else
				show_error_msg("ERRORE: spiacente hai raggiunto il numero massimo di nominativi");
		} else {
			show_error_msg("ERRORE: non ho potuto completare l'operazione di aggiunta campo. Riprova oppure contatta l'autore.");
			g_print(" cod = %d > cards = %d", cod, Status[S_CARDS]);
		}
	} else
		show_error_msg("ERRORE: nessuna rubrica  aperta: seleziona la voce APRI dal men FILE.");
}


void on_elimina_activate(GtkMenuItem * menuitem, gpointer user_data)
{

}


void on_info_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	int i;
	char ch;

	stampa_infos();
	if (NomeFile != NULL)
		g_print("\nNomefile: %s", NomeFile);
	if (SoloNomeFile != NULL)
		g_print("\nSoloNomefile: %s", SoloNomeFile);

	if (Status[S_CARDS] > 0) {
		g_print("\nFirst 100 chars of current card:\n");
		for (i = 0; i < 100; i++) {
			ch = Lista[Status[S_SELISTA]]->cod[i];
			if (ch)
				g_print("%c", ch);
			else
				g_print("\\0");
		}
	}

	if (!Status[S_INFO]) {
		show_info_msg();
		Status[S_INFO] = 1;
	}
}


void on_button_aggiungi_clicked(GtkButton * button, gpointer user_data)
{
	on_aggiungi_activate(NULL, user_data);
}


void on_ricerca_changed(GtkEditable * editable, gpointer user_data)
{
	int k;
	GtkWidget *entry;
	gchar *strn;

	g_print("\nRicerca Text Changed");
	if (Status[S_INIBIT] == S_RICERCA) {
		g_print("\nRicerca inibit");
		Status[S_INIBIT] = -1;
		return;
	}

	entry = lookup_widget(GTK_WIDGET(Base), "ricerca");
	strn = g_strdup_printf("%s", gtk_entry_get_text(GTK_ENTRY(entry)));

	if (!strcmp((char *) strn, ""))
		Status[S_RICERCA] = 0;
	else {
		k = trova_ricerca((const char *) strn);
		g_print("list %d find as first (%d, %d)", k, Status[S_RICERCA], Status[S_CARDS]);

		if (Status[S_RICERCA] == 2 && k > -1 && Status[S_CARDS] > k) {
			g_print("\nPrinting card");
			set_lista_item(k, !S_INIBIT);
		} else if (k > -1)
			set_lista_item(k, S_INIBIT);

		Status[S_RICERCA] = 1;

		/* 
		Queste constanti sono soggette al cambiamento rispetto all'uso di caratteri 
		diversi per questo motivo la funzione set_vscroll_windows non  utilizzabile 
		in genere ma va sostituita con set_scrolled_windows 
		 
		set_vscroll_window(Base, "scrolledwindow4", V_START_LIST + k*V_STEP_LIST); 
		*/
	}
	g_free(strn);
}


void on_ricerca_insert_text(GtkEditable * editable, gchar * new_text, gint new_text_length, gint * position, gpointer user_data)
{
	static int inserit = 0;

	inserit++;
	g_print("\nRicerca Text was inserit %d times", inserit);
}


gboolean on_ricerca_key_press_event(GtkWidget * widget, GdkEventKey * event, gpointer user_data)
{
	int key;

	key = get_whats_key(event);
	g_print("\nKey 0x%x pressed on Ricerca Box", key);

	switch (key) {

		// case KEYB_FDOWN: Status[S_INIBIT] = KEYB_FDOWN;
		// set_lista_item(Status[S_SELISTA], S_INIBIT);

	case KEYB_INVIO:
		Status[S_RICERCA] = 2;			// permette di stampare la card trovata 
		on_ricerca_changed(NULL, user_data);
		break;
	}

	return FALSE;
}



void on_lista_select_child(GtkList * list, GtkWidget * widget, gpointer user_data)
{
	int cd = 0, i, cont;
	static int lchildcount = 0;
	char strn[100];
	gchar *item_data_string;

	/* 
	La selection changed avviene 2 volte per click: 
 	una per deselezione ed una per selezione 
 	*/

	/* Non conta quelle volte che viene chiamata dal prg */
	lchildcount++;
	g_print("\nLista Selection Child");
	if (Status[S_INIBIT] == S_LISTA) {
		g_print(" ignored by S_LISTA");
		return;
	}
	/* 
	 * else if( Status[S_INIBIT] == KEYB_FDOWN) * { * Status[S_INIBIT] = -1; * g_print (" correction by down arrow"); *
	 * set_lista_item(Status[S_SELISTA], !S_INIBIT); * return; * } 
	 */


	if (Status[S_LISTA] > 0) {
		item_data_string = gtk_object_get_data((list->selection)->data, list_item_data_key);
		cd = atoi(&item_data_string[1]);
		g_print(" %s (%d)", &item_data_string[1], cd);

		/* 
		Un'altro modo di farlo
 		GtkWidget * lst;
 		GList * dlist;
 		GtkObject * list_item;

 		list_item = get_selected_list_item(Base, "lista");
 		item_data_string = gtk_object_get_data(list_item, list_item_data_key); 
 		cd = atoi( &item_data_string[1] ); 
 		g_print(" %s (%d)", &item_data_string[1], cd);
 		*/

		cont = 1;
		for (i = 0; i < Status[S_CARDS] && cont; i++)
			if (Lista[i]->num == cd) {
				Status[S_SELISTA] = i;
				g_print(" - %d", i);

				//gtk_list_start_selection(list);
				//gtk_list_toggle_focus_row(list);
				//gtk_window_set_focus (widget->window, widget);
				//gtk_widget_grab_focus(widget);
				set_scrolled_window(Base, "scrolledwindow4", "lista", ON_PAGE);

				/* 
				Nell'impostare la ricerca si attiverebbe il segnale 
 				on_ricerca_changed impropriamente quindi mi occupo
 				di disattivarlo manualmente
 				*/
				if (Status[S_INIBIT] < 1)
					Status[S_INIBIT] = S_RICERCA;
				else
					g_print("\nERROR: INIBIT not free");

				g_print("\nStatus[S_RICERCA] = %d, ", Status[S_RICERCA]);
				if (abs(Status[S_ORDINE]) == ORD_COG)
					sprintf(strn, "%s %s", Lista[i]->cog, Lista[i]->nom);
				else
					sprintf(strn, "%s %s", Lista[i]->nom, Lista[i]->cog);
				set_ricerca(strn);
				g_print("\nStatus[S_RICERCA] = %d, %d", Status[S_RICERCA], cd + 1);
				Status[S_RICERCA] = cd + 1;
				VISUALIZZA_CARD(i);
				cont = 0;
			}
	}
}


void on_radio_cognome_toggled(GtkToggleButton * togglebutton, gpointer user_data)
{
	if (abs(Status[S_ORDINE]) != ORD_COG) {
		if (ORD_VTEST(Status[S_ORDINE]) != ORD_INV)
			Status[S_ORDINE] = ORD_ALF * ORD_COG;
		else
			Status[S_ORDINE] = ORD_INV * ORD_COG;

		g_print("\nRadio button toggled");
		if (Status[S_RUBRICA])
			imposta_lista();
		g_print("\nOrdinanamento per cognome (%d,%d)", Status[S_ORDINE], Status[S_RUBRICA]);
	}
}


void on_radio_alfabetico_toggled(GtkToggleButton * togglebutton, gpointer user_data)
{
	if (ORD_VTEST(Status[S_ORDINE]) == ORD_INV) {
		Status[S_ORDINE] *= ORD_REV;

		g_print("\nRadio button toggled");
		if (Status[S_RUBRICA]) {
			imposta_lista();
			// set_lista_item(Status[S_SELISTA], !S_INIBIT);
		}
		g_print("\nOrdinanamento alfabetico (%d,%d)", Status[S_ORDINE], Status[S_RUBRICA]);
	}
}


void on_radio_nome_toggled(GtkToggleButton * togglebutton, gpointer user_data)
{
	if (abs(Status[S_ORDINE]) != ORD_NOM) {
		if (ORD_VTEST(Status[S_ORDINE]) != ORD_INV)
			Status[S_ORDINE] = ORD_ALF * ORD_NOM;
		else
			Status[S_ORDINE] = ORD_INV * ORD_NOM;

		g_print("\nRadio button toggled");
		if (Status[S_RUBRICA])
			imposta_lista();
		g_print("\nOrdinanamento per nome (%d,%d)", Status[S_ORDINE], Status[S_RUBRICA]);
	}
}


void on_radio_inverso_toggled(GtkToggleButton * togglebutton, gpointer user_data)
{
	if (ORD_VTEST(Status[S_ORDINE]) != ORD_INV) {
		Status[S_ORDINE] *= ORD_REV;

		g_print("\nRadio button toggled");
		if (Status[S_RUBRICA]) {
			imposta_lista();
			// set_lista_item(Status[S_SELISTA], !S_INIBIT);
		}
		g_print("\nOrdinanamento inverso (%d,%d)", Status[S_ORDINE], Status[S_RUBRICA]);
	}
}


void on_base_destroy(GtkObject * object, gpointer user_data)
{
	/* Passando per il destroy occore ricreare la finestra di base */
	Status[S_AVVRTM] = AVT_DESTROYB;

	/* poi si ritorna al ciclo standard */
	on_esci_activate(NULL, user_data);
}


void on_info_released(GtkButton * button, gpointer user_data)
{
	/* 
	Dovrebbe essere distrutto altrimenti ci troviamo ocn un memory leak
 	gtk_widget_hide (GTK_WIDGET (Info));
 	*/
	gtk_widget_destroy(GTK_WIDGET(Info));
	Status[S_INFO] = 0;
}


void on_avvertimento_ritorna_clicked(GtkButton * button, gpointer user_data)
{
	char text[TXT_LUN];

	gtk_widget_hide(Avvertimento);
	switch (Status[S_AVVRTM]) {
	case AVT_DESTROYB:
		CREATE_BASE(Base);
		gtk_widget_set_usize(Base, Wbase.w, Wbase.h);
		sprintf(text, "Rubrica: \"%s\"", SoloNomeFile);
		g_set_window_title(Base, text);
		imposta_lista();
		gtk_widget_show(Base);
	case AVT_BASE:
		gtk_widget_set_sensitive(Base, TRUE);
		break;

	case AVT_DESTROYC:
		Creazione = create_creazione();
		ritorna_modello();
		gtk_widget_show(Creazione);
	case AVT_CREAZION:
		break;

	case AVT_DESTROYM:
		modifica_ripristina();
	case AVT_MODIFICA:
		gtk_widget_show(Modifica);
		gtk_widget_set_sensitive(Modifica, TRUE);
		break;

	case AVT_APRI:
		gtk_widget_set_sensitive(Base, TRUE);
		break;

	default:
		g_print("\nERROR: AVVRTM Status (%d) not assigned!", Status[S_AVVRTM]);
		break;
	}

	/* Per eliminare l'avvertimento devo inibire il suo ritorno */
	g_print("\nDistruggo l'avvertimento (%d)", Status[S_AVVRTM]);
	Status[S_AVVRTM] = AVT_INIBITO;
	gtk_widget_destroy(Avvertimento);

	/* Pulisce lo stato di avvertimento (in particolare DESTROY se esiste) */
	Status[S_AVVRTM] = 0;
}


void on_avvertimento_rigetta_clicked(GtkButton * button, gpointer user_data)
{
	gtk_widget_hide(Avvertimento);
	switch (Status[S_AVVRTM]) {
	case AVT_DESTROYB:
		/* 
		Anche se devo uscire, creare la finestra base 
		permette una gtk_exit pulita e corretta 
		*/
		if (Base == NULL)
			CREATE_BASE(Base);
	case AVT_BASE:
		stampa_infos();
		save_init_values();
		g_print("\n");
		gtk_exit(0);
		break;

	case AVT_CREAZION:
		g_print("Annulla modifiche creazione");
		if (Creazione != NULL) {
			gtk_widget_destroy(Creazione);
			Creazione = NULL;
		}
	case AVT_DESTROYC:
		Status[S_LISTA] = (Status[S_CARDS]) ? 1 : 0;
		ANNULLA_MODIFICHE_CREA();
	case AVT_MODIFICA:
		if (Modifica != NULL) {
			gtk_widget_destroy(Modifica);
			Modifica = NULL;
		}
	case AVT_DESTROYM:
		ANNULLA_MODIFICHE_CAMPI();
		gtk_widget_set_sensitive(Base, TRUE);
		g_print("\nStato di alterazione: %d", Status[S_CRDMOD]);
		break;

	case AVT_APRI:
		if (FileDialog == NULL)
			FileDialog = create_file_dialog();
		gtk_window_set_title(GTK_WINDOW(FileDialog), "Apertura di un file");
		MOSTRA_ON_MOUSE(FileDialog);
		break;

	default:
		g_print("\nERROR: AVVRTM Status (%d) not assigned!", Status[S_AVVRTM]);
		break;
	}

	/* Per eliminare l'avvertimento devo inibire il suo ritorno */
	Status[S_AVVRTM] = AVT_INIBITO;
	gtk_widget_destroy(Avvertimento);

	/* Pulisce lo stato di avvertimento (in particolare DESTROY se esiste) */
	Status[S_AVVRTM] = 0;
}


void on_avvertimento_destroy(GtkObject * object, gpointer user_data)
{
/*
 * Distruggere il msg di avvertimento equivale a mantenere le modifiche
 * ma occorre distinguere il caso in cui sia l'utente a distruggere il
 * messaggio oppure se si tratta del programma per necessita` 
 */
	if (!(Status[S_AVVRTM] & AVT_INIBITO)) {
		g_print("\nAvvertimento distrutto (%d): ritorno ai dati non salvati", Status[S_AVVRTM]);
		on_avvertimento_ritorna_clicked(NULL, user_data);
		Status[S_AVVRTM] = 0;
	}
}


void on_ok_errormsg_pressed(GtkButton * button, gpointer user_data)
{
	Msg_Errore = lookup_widget(GTK_WIDGET(button), "errore");
	gtk_widget_destroy(Msg_Errore);
}


gboolean on_base_drag_drop(GtkWidget * widget, GdkDragContext * drag_context, gint x, gint y, guint time, gpointer user_data)
{
	g_print("\nDrag'n'Drop activated:\n");

	return FALSE;
}


void on_trova_prossimo_activate(GtkMenuItem * menuitem, gpointer user_data)
{

}


gboolean on_ricerca_button_press_event(GtkWidget * widget, GdkEventButton * event, gpointer user_data)
{
	static int pippo = 0;

	pippo++;
	g_print("\nOn ricerca click catched (%d): %d", pippo, (int) event->button);

	return FALSE;
}


void on_telefona_clicked(GtkButton * button, gpointer user_data)
{
	GtkWidget *toggle;

	g_print("\nTelefono: %d", Status[S_TELEFONO]);
	if (!Status[S_TELEFONO]) {
		Status[S_TELEFONO] = 1;
		Telefono = create_telefono();
		MOSTRA_ON_MOUSE(Telefono);

		toggle = lookup_widget(GTK_WIDGET(Telefono), "terminal");
		g_print("\nButton toggled from %d to %d", Status[S_MTERM], GTK_TOGGLE_BUTTON(toggle)->active);
		if (GTK_TOGGLE_BUTTON(toggle)->active)
			Status[S_MTERM] = 1;
		else
			Status[S_MTERM] = 0;
	}
}


void on_button_modella_clicked(GtkButton * button, gpointer user_data)
{
	creazione_modello();
}


void on_modella_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	creazione_modello();
}


void on_button_rinomina_clicked(GtkButton * button, gpointer user_data)
{
	on_salva_con_nome_activate(NULL, NULL);
}


void on_minuscolo_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	FixedFont = gdk_fontset_load("-misc-fixed-medium-r-*-*-*-80-*-*-*-*-*-*");
	VISUALIZZA_CARD(Status[S_SELISTA]);
	set_init_value(ILV_FIXED_FONT, "-misc-fixed-medium-r-*-*-*-80-*-*-*-*-*-*");
}


void on_piccolo_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	FixedFont = gdk_fontset_load("-misc-fixed-medium-r-*-*-*-90-*-*-*-*-*-*");
	VISUALIZZA_CARD(Status[S_SELISTA]);
	set_init_value(ILV_FIXED_FONT, "-misc-fixed-medium-r-*-*-*-90-*-*-*-*-*-*");
}


void on_medio_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	FixedFont = gdk_fontset_load("-misc-fixed-medium-r-*-*-*-100-*-*-*-*-*-*");
	VISUALIZZA_CARD(Status[S_SELISTA]);
	set_init_value(ILV_FIXED_FONT, "-misc-fixed-medium-r-*-*-*-100-*-*-*-*-*-*");
}


void on_grosso_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	FixedFont = gdk_fontset_load("-misc-fixed-medium-r-*-*-*-120-*-*-*-*-*-*");
	VISUALIZZA_CARD(Status[S_SELISTA]);
	set_init_value(ILV_FIXED_FONT, "-misc-fixed-medium-r-*-*-*-120-*-*-*-*-*-*");
}


void on_enorme_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	FixedFont = gdk_fontset_load("-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*");
	VISUALIZZA_CARD(Status[S_SELISTA]);
	set_init_value(ILV_FIXED_FONT, "-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*");
}


void on_cerca_ovunque_activate(GtkMenuItem * menuitem, gpointer user_data)
{

}


void on_buttonT_clicked(GtkButton * button, gpointer user_data)
{

}


void on_base_check_resize(GtkContainer * container, gpointer user_data)
{
	char *strn;
	int x[2], w, h;

	x[0] = (int) ((char *) container)[36] & 0xff;
	x[1] = (int) ((char *) container)[37] & 0xff;
	w = x[0] + x[1] * 256;
	x[0] = (int) ((char *) container)[38] & 0xff;
	x[1] = (int) ((char *) container)[39] & 0xff;
	h = x[0] + x[1] * 256;
	g_print("\n Resizing: %d, %d", w, h);

  	//ispezione((char *)container); 
 	
	if (w < X_MIN_BASE_SIZE || h < Y_MIN_BASE_SIZE) {
		Wbase.w = (w < X_MIN_BASE_SIZE) ? X_MIN_BASE_SIZE : w;
		Wbase.h = (h < Y_MIN_BASE_SIZE) ? Y_MIN_BASE_SIZE : h;
		g_print("\n Ritorno alle dimensioni minime ( w = %d, h = %d )", Wbase.w, Wbase.h);
		((char *) container)[36] = (char) (Wbase.w % 256);
		((char *) container)[37] = (char) (Wbase.w / 256);
		((char *) container)[38] = (char) (Wbase.h % 256);
		((char *) container)[39] = (char) (Wbase.h / 256);
	} else {
		Wbase.w = w;
		Wbase.h = h;
	}

	/* Setto la label sulla dimensione della finestra */
	strn = strdup("");
	sprintf(strn, "%dx%d", Wbase.w, Wbase.h);
	set_init_value(ILV_BASE_SIZE, strn);
}


void on_esporta_dest_clicked(GtkButton * button, gpointer user_data)
{

}


void on_option_esportadest_menu_selected(GtkMenuShell * menu_shell, gpointer user_data)
{
	GtkWidget *active_item;
	gint item_index;

	active_item = gtk_menu_get_active(GTK_MENU(menu_shell));
	item_index = g_list_index(menu_shell->children, active_item);
}


void on_esp_option_ncards_clicked(GtkButton * button, gpointer user_data)
{

}


void on_option_espoptionncards_menu_selected(GtkMenuShell * menu_shell, gpointer user_data)
{
	GtkWidget *active_item;
	gint item_index;

	active_item = gtk_menu_get_active(GTK_MENU(menu_shell));
	item_index = g_list_index(menu_shell->children, active_item);
}
