/**************************************************************/
/***                                                        ***/
/*** Gestione dei files: "apri", "salva" e "salva con nome" ***/
/***                                                        ***/
/**************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifndef SUB_CALLING
#include <stdio.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>

#include "interface.h"
#include "support.h"
#include "callbacks.h"
#include "pfunc.h"
#include "sfunc.h"
#include "iofile.h"
#endif


void
on_button_apri_clicked (GtkButton * button, gpointer user_data)
{
	on_apri_activate (NULL, user_data);
}


void
on_apri_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	Status[S_FILEDIALOG] = FILE_APRI;

	if (Status[S_CRDMOD] || Status[S_MODIFICA])
		chiudi_con_modifiche (Base, AVT_APRI);
	else
	{
		if (FileDialog == NULL)
			FileDialog = create_file_dialog ();
		gtk_window_set_title (GTK_WINDOW (FileDialog),
				      "Apertura di un file");
		gtk_widget_set_sensitive (Base, FALSE);
		MOSTRA_ON_MOUSE (FileDialog);
	}
}


void
on_button_salva_clicked (GtkButton * button, gpointer user_data)
{
	on_salva_activate (NULL, user_data);
}


void
on_salva_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	char *str;
  //char *cmp[MAX_N_CAMPI];
	//int i; 
	int j;
	FILE *fp;

	if (!Status[S_RUBRICA])
		show_error_msg
			("ERRORE: nessuna rubrica da salvare: seleziona la voce APRI dal men FILE.");
	else if (!Status[S_MODIFICA] && !Status[S_FILEDIALOG])
		show_error_msg
			("ATTENZIONE: mi rifiuto di salvare perche` non vi sono state modifiche");
	else
	{
		/* La possibilita` di fare il back andrebbe sondata allapertura
		 * del file in modo da avvisare tempestivamente lutente
		 * La seconda ragione e` per evitare che una sola modifica salvata
		 * piu` volte generi backup multipli inutili per questom otivo
		 * il backup non e` ripetuto piu` di una volta a sessione  */

		/**************************************************/
		/* Inizio la procedura di backup del vecchio file */
		/**************************************************/

		/* Se salvo con nome non faccio il backup ne chiedo se posso
		 * sovvrascrive il file di arrivo. Entrambe queste due possibilita`
		 * devono essere ridimensionate per evitare la perdita dati */

		/* Inizializza j come se il backup fosse fallito 
		 * ma non se "salva con nome" */
		j = (int) (!Status[S_FILEDIALOG]);
		if (!Status[S_BACKUP] && !Status[S_FILEDIALOG])
		{
			str = file_backup_name (NomeFile);

			/* Se il file non esiste non serve farne il backup */
			if (str != NULL)
			{
				j = file_copy (NomeFile, str);
				g_print ("\nCopiato %s in %s (%d)", NomeFile,
					 str, j);
				M_FREE (str);
			}
			else
				Status[S_BACKUP] = -1;
		}

		/* La prima volta se ci riesce fa il backup, cmq salva il file
		 * Le altre volte procede a salvare il file e basta */
		if (!j || Status[S_BACKUP])
		{
			g_print ("\nNomeFile aperto: %s", NomeFile);
			fp = fopen (NomeFile, "w");
			Status[S_BACKUP] = 1;
		}
		else
		{
			g_print ("\nNomeFile aperto (%d, %d): %s.bak",
				 Status[S_BACKUP], j, NomeFile);

			Status[S_BACKUP]++;
			str = (char *) malloc (strlen (NomeFile) + 8);
			sprintf (str, "%s.bak", NomeFile);
			fp = fopen (str, "w");
			M_FREE (str);
		}

  	/*********************************************/
		/* Inizio la scrittura della rubrica su file */
  	/*********************************************/

		/* RAF 2001-06-04: il formato base  il CSV + Sequenza */
		esporta_rubrica_in_csv(fp);

		if(Status[S_FILEDIALOG] == FILE_SALVA) 
		{
			/* RAF 2001-06-04: non serve pi 
			for (i = 0; i < Status[S_CAMPI]; i++)
				if (Campi[i] != NULL)
					fprintf (fp, "\"%s\"\t", Campi[i]);
			fprintf (fp, "\n");

			for (i = 0; i < Status[S_CARDS]; i++)
			{
				estrai_campi_da_card (Lista[i], cmp);
				for (j = 0; j < Status[S_CAMPI]; j++)
					if (Campi[j] != NULL)
						fprintf (fp, "\"%s\"\t",
							 (strlen (cmp[j]) <=
							  0) ? "_" : cmp[j]);
				fprintf (fp, "\n");
			}
			*/
			replace_in_sequenza ("\n", "\\n", FR_STRINGHE | FR_CASE_SEN);
			fprintf (fp, "%s%c%c%c", Sequenza, 0, 0, 0);
		} 
		else if(Status[S_FILEDIALOG] == FILE_ESPORTA) 
		{
			/* RAF 2001-06-04: ripristino il vecchio nome file */
			M_FREE(NomeFile);     
      NomeFile = M_STRDUP(OldNomeFile);
			M_FREE(SoloNomeFile); 
      SoloNomeFile = M_STRDUP(OldSoloNomeFile);
		}

		fclose (fp);
		if(Status[S_FILEDIALOG] == FILE_SALVA) 
		{
			/* La rubrica ormai  salvata */
			Status[S_CRDMOD] = 0;
			Status[S_MODIFICA] = 0;
			set_sensitive_tools (IF_NEED);
		}
	}
}


void
on_apri_ok_button_clicked (GtkButton * button, gpointer user_data)
{
	char text[TXT_LUN];
	char *nome;
	int i;

	g_print ("\nOk pressed (%d), file selected: ", Status[S_FILEDIALOG]);
	nome =
		gtk_file_selection_get_filename (GTK_FILE_SELECTION
						 (FileDialog));
	M_FREE (NomeFile);
	NomeFile = M_STRDUP (nome);
	g_print ("%s", NomeFile);

	gtk_widget_hide (GTK_WIDGET (FileDialog));
	gtk_widget_set_sensitive (Base, TRUE);

	if (Status[S_FILEDIALOG] == FILE_APRI)
	{
		/* Se ho usato la modellazione per evitare che i vecchi campi
		 * continuino ad infestare la listacampi elimino la finestra  */
		if (Creazione != NULL)
		{
			gtk_widget_destroy (Creazione);
			Creazione = NULL;
		}

		Status[S_CRDMOD] = 0;
		Status[S_MODIFICA] = 0;
		g_print ("\nStato di variazione resettato %d",
			 Status[S_CRDMOD] | Status[S_MODIFICA]);
		ApriFileByName (NomeFile);
	}
	else if (Status[S_FILEDIALOG] == FILE_SALVA ||
	         Status[S_FILEDIALOG] == FILE_ESPORTA)
	{
		for (i = strlen (NomeFile) - 1; i > 0; i--)
			if (NomeFile[i] == '/')
				break;
		if (i)
			i++;
		M_FREE (SoloNomeFile);
		SoloNomeFile = M_STRDUP (&(NomeFile[i]));

		/* Il file ha assunto un nuovo nome e il titolo
		 * della finestra deve essere aggiornato */
		/* str = (char *)malloc(strlen(SoloNomeFile) + 16); */
		sprintf (text, "Rubrica: \"%s\"", SoloNomeFile);
		g_set_window_title (Base, text);
		g_print ("\nTitolo di rubrica: -%s-", text);

		on_salva_activate (NULL, NULL);
	}

	Status[S_FILEDIALOG] = 0;
}


void
on_apri_cancel_button_clicked (GtkButton * button, gpointer user_data)
{
	g_print ("\nCancel pressed");
	gtk_widget_hide (GTK_WIDGET (FileDialog));
	gtk_widget_set_sensitive (Base, TRUE);
	Status[S_FILEDIALOG] = 0;
}


void
on_salva_con_nome_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	gtk_widget_set_sensitive (Base, FALSE);
	Status[S_FILEDIALOG] = FILE_SALVA;
	if (FileDialog == NULL)
		FileDialog = create_file_dialog ();
	gtk_window_set_title (GTK_WINDOW (FileDialog), "Salvataggio del file");
	MOSTRA_ON_MOUSE (FileDialog);
}


void
on_esporta_csv_activate (GtkMenuItem * menuitem, gpointer user_data)
{	
	/* RAF 2001-06-04: salvo il vecchio nome file perch 
	   l'esportazione non  considerato salvataggio 
   */
	M_FREE(OldNomeFile);
	OldNomeFile = strdup(NomeFile);
	M_FREE(OldSoloNomeFile);
	OldNomeFile = strdup(SoloNomeFile);

	gtk_widget_set_sensitive (Base, FALSE);
	Status[S_FILEDIALOG] = FILE_ESPORTA;
	if (FileDialog == NULL)
		FileDialog = create_file_dialog ();
	gtk_window_set_title (GTK_WINDOW (FileDialog), "Esportazione CSV del file");
	MOSTRA_ON_MOUSE (FileDialog);
}



void
on_file_dialog_destroy (GtkObject * object, gpointer user_data)
{
	FileDialog = NULL;
	gtk_widget_set_sensitive (Base, TRUE);
}


void
on_file_dialog_show (GtkWidget * widget, gpointer user_data)
{

}
