#ifdef HAVE_CONFIG_H
 #include <config.h>
#endif

#ifndef SUB_CALLING
 #include <stdio.h>
 #include <gtk/gtk.h>
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
 
 #include "interface.h"
 #include "support.h"
 #include "callbacks.h"
 #include "pfunc.h"
 #include "sfunc.h"
 #include "mfunc.h"
 #include "iofile.h"
#endif


void ApriFileByName(char * const name)
{
 int len, k, i;
 char text[TXT_LUN];

 if ( !access(name,R_OK | F_OK) )
 {
  g_print ("\nTry to open as input files: ");

  if( name != NomeFile )
  {
   M_FREE(NomeFile);
   NomeFile = M_STRDUP(name);
  }
  len = strlen(NomeFile);

  /* Comincio una serie di controlli sulla validita` del file selezionato */
  if( NomeFile == NULL || NomeFile[len-1] == '/')	
   g_print ("none");					/* Le directory sono escluse */
  else
  {
   g_print ("%s", NomeFile);

   k = 0;
   for (i = 0; i < strlen(NomeFile); i++)
   {
    if (NomeFile[i] == '/' )
     k = i;
   }
   if ( k ) k++;

   M_FREE(SoloNomeFile); SoloNomeFile = M_STRDUP( &(NomeFile[k]) );

   sprintf(text,"Rubrica: \"%s\"", SoloNomeFile);
   g_set_window_title (Base, text);

   sprintf(text,"Reading %s", NomeFile);
   g_set_statusbar (Base, text);

   /* Leggo il file e controllo che sia letto correttamente */
   if( !leggi_rubrica() )				
   {
    sprintf(text,"Loading %s", NomeFile);
    g_set_statusbar (Base, text);

    carica_rubrica();
    if (Status[S_RICERCA])
    {
     Status[S_INIBIT] = S_RICERCA;
     set_ricerca("");
    }
    /* Questa riga risolve il problema della modifica 
       del primo campo selezionato ma non stampato */
    set_lista_item(0, !S_INIBIT);

    sprintf(text,"Nominativi inseriti: %d ",Status[S_CARDS]);
    g_set_statusbar (Base, text);

    if (Status[S_CAMPI] < 1)
     g_print("\nERROR: acquisizione campi di intestazione riporta %d",k);
   }
  }
 }
 else
  g_print("\nFile %s non esiste oppure non hai il permesso per leggerlo", name);
}


/* In caso di errore nell'aprire il file
   Occorre inizializzare Rubrica Italiana
   come se la si fosse appena lanciata */
#define LR_ERROR_EXIT(testo) {		\
  fclose(fp);				\
  RESTART_BASE();			\
  show_error_msg(testo);		\
}					
int leggi_rubrica(void)
{
 FILE * fp;
 size_t lu;
 long l;

 g_print ("\nReading rubrica file (continua)");

 fp = fopen(NomeFile, "rt");   				/* apre per leggere */
 FileBufferLun = (unsigned long)getfilelenght(fp);	/* misura la lunghezza del file */
 g_print (" of %ld bytes lenght", FileBufferLun);

 free_memory();

 if (MAX_FILE_LUN < FileBufferLun)			/* controlla che la lunghezza del file */
 {							/* sia inferiore a quella massima consentita */
  FileBuffer = (char *)malloc(MAX_FILE_LUN+10);
  g_print (" exceed");

  LR_ERROR_EXIT("ERRORE: il file selezionato ha una dimensione troppo grande.");
  return -1;
 }
 else
 {
  FileBuffer = (char *)malloc(FileBufferLun+10);	/* alloca la memoria per leggere la rubrica +10 bytes*/
  if(FileBuffer != NULL)				/* Controlla che effettivamente l'allocazione */
   g_print (" allocated\n");				/* della memoria sia riuscita */
  else
  {
   LR_ERROR_EXIT("ERRORE: non riesco ad allocare la memoria necessaria");
   return -2;
  }

  lu = fread(FileBuffer, 1, FileBufferLun, fp);		/* legge il file che contiene la rubrica */
  if (lu != (size_t)FileBufferLun)			/* controlla che sia riuscita la lettura */
  {
   LR_ERROR_EXIT("ERRORE: non riesco a leggere il file selezionato.");
   return -3;
  }
  else							/* un piccolo trucco che evita casini */
   FileBuffer[lu] = 0;

  l = find_first(FileBuffer, "#seq: \"", IGN_CASE | REVERSE);
  if(l < 0L)
  {
   LR_ERROR_EXIT("ERRORE: il file selezionato non sembra essere una rubrica valida.");
   return -4;
  }
 }

 /* Chiudo il file dopo averne copiato il
    contenuto in memoria ed esco senza errore */
 Status[S_BUFFER] = 1;
 fclose(fp);    					

 return 0;
}


char * file_backup_name(const char * nomefile)
{
 int i, status;
 char * str1, * str2;

 i = 0;
 str1 = M_STRDUP(nomefile);
 if( access(str1, F_OK) )
  return NULL;

 do {
  str2 = (char *)malloc(strlen(str1) + 5);
  sprintf(str2,"%s.bak", str1);
  M_FREE(str1);
  str1 = str2;
  status = access(str1, F_OK);
  i++;
 }
 while( !status && i < MAX_BACK_COUNT);

 if( i >= MAX_BACK_COUNT )
 {
  str2 = (char *)malloc(strlen(str1) + 5);
  sprintf(str2,"-f %s", str1);
  M_FREE(str1);
 }
 return str2;
}


int file_copy(char * const nomefile, char * const backfile)
{
 int status;
 char * str;

 if( access( nomefile, R_OK ) )
 {
  /* Con le nuove funzioni di "Salva con nome" e di creazione
     della rubrica questo allarme non e` piu` sostenibile
     show_error_msg("\nERRORE: nel fare il backup il file sorgente non  leggiible"); */
  return -1;
 }

 g_print("\nBackup %s %s", nomefile, backfile);
 str = (char *)malloc(strlen(nomefile) + strlen(backfile) + 8);
 sprintf(str, "cp %s %s", nomefile, backfile);
 status = system(str);
 M_FREE(str);

 if(status == 127 || status == -1)
  return -3;
 else
  return 0;
}


unsigned long getfilelenght(FILE * fp)
{
 long lun;

 fseek( fp, 0, SEEK_END);
 lun = ftell(fp);
 rewind(fp);

 return lun;
}

/************ FUNZIONI DI INIZIALIZZAZIONE **********/

/*  Per semplificare lo sviluppo da parte di pi persone
   e permettere a Rubrica Italiana di inizializzare vari
   parametri del'interfaccia utente ho deciso di creare
   un set di funzioni tali che ognuno possa definire una
   label e un valore ad esso associato.
    In questo modo sar sufficiente per un programmatore
   affidare ad una label (unica) il valore da salvare e
   tramite questa label riottenere il valore.
    Non potendo conooscere il type migliore per salvare
   il valore e considerato che comunque questo avrebbe
   dovuto essere scritto su un file di testo il valore
    una stringa (e il set di tale valore si occupa di
   duplicarla) */


int set_init_value(const char * label, const char * value)
{
 int i;

 if(label == NULL || value == NULL)
 {
  g_print("\nERROR: Init value and label cannot be NULL");
  return 0;
 }

 for (i = 0; i < MAX_INIT_LABEL; i++)
  if(InitVal[i].label != NULL)
   if(!strcmp(InitVal[i].label, label))
    break;

 if(i < MAX_INIT_LABEL)
 {
  /* La label esiste gi e deve esserne aggiornato il valore */
  M_FREE(InitVal[i].value);
  InitVal[i].value = strdup(value);
  i = -i-1;
 }
 else
 {
  /* La label deve essere creata ed inizializzata */
  for (i = 0; i < MAX_INIT_LABEL; i++)
   if(InitVal[i].label == NULL)
    break;
  InitVal[i].label = strdup(label);
  InitVal[i].value = strdup(value);
  i++;
 }

 /* Ritorna l'indice+1 del valore modificato (negativo) oppure
    ritorna l'indice+1 del valore creato (positvo) cos che
    il valore zero rimane per gli errori */
 return i;
}


char * get_init_value(const char * label)
{
 int i;

 if(label == NULL)
 {
  g_print("\nERROR: Init label cannot be NULL");
  return NULL;
 }

 for (i = 0; i < MAX_INIT_LABEL; i++)
  if(InitVal[i].label != NULL)
   if(!strcmp(InitVal[i].label, label))
    break;

 if(i < MAX_INIT_LABEL)
  return strdup(InitVal[i].value);
 else
  return NULL;
}


int del_init_value(const char * label)
{
 int i;

 if(label == NULL)
 {
  g_print("\nERROR: Init label cannot be NULL");
  return -2;
 }

 for (i = 0; i < MAX_INIT_LABEL; i++)
  if(InitVal[i].label != NULL)
   if(!strcmp(InitVal[i].label, label))
    break;

 if(i < MAX_INIT_LABEL)
 {
  M_FREE(InitVal[i].label);
  M_FREE(InitVal[i].value);
 }
 else
  i = -1;

 /* Ritorna l'indice del valore eliminato oppure
    se non trova la label ritorna il valore -1 */
 return i;
}


int load_init_values(void)
{
 unsigned long flun;
 unsigned long flrd;
 FILE * file;
 char * buf;

 if ( access(InitFileName, R_OK | F_OK) )
 {
  g_print("\nERROR: I cannot access init file %s", InitFileName);
  return -2;
 }
 file = fopen(InitFileName, "rt");
 if(file != NULL)
 {
  flun = (unsigned long)getfilelenght(file);
  buf = (char *)malloc(flun+16);
  if(buf != NULL)
  {
   flrd = fread(buf, 1, flun, file);
   buf[flrd] = 0;
   fclose(file);

   read_init_values(buf);
   M_FREE(buf);

   return 0;
  }
  else
   g_print("\nERROR: I cannot malloc buffer");
 }
 else
  g_print("\nERROR: I cannot open init file %s", InitFileName);

 return -1;
}


int save_init_values(void)
{
 int i, n = 0;
 FILE * file;

 /* Se non ci sono label inutile salvare */
 for (i = 0; i < MAX_INIT_LABEL; i++)
  if(InitVal[i].label != NULL)
   break;
 if( i >= MAX_INIT_LABEL)
  return 0;

 g_print("\nTrying to write init file %s", InitFileName);
 file = fopen(InitFileName, "wt");
 if(file != NULL)
 {
  fprintf(file, "#\n# %s\n# %s\n# %s\n# %s\n#", RVERSN, AUTHOR, EMAIL, WEBPAGE);

  for (i = 0; i < MAX_INIT_LABEL; i++)
   if(InitVal[i].label != NULL)
   {
    fprintf(file, "\n%s = %s", InitVal[i].label, InitVal[i].value);
    n++;
   }
  fprintf(file, "\n");

  g_print(": succes");
  fclose(file);
  return n;
 }
 else
  g_print(": failed");

 return -1;
}


/* Questa funzione  utilizzata esclusivamente
   dalla funzione load_init_values(); */
int read_init_values(char * buf)
{
 int lun, com, j, k, i = 0, n = 0, start = 0, max = 0;
 char * sln[MAX_INIT_LABEL], * sln1, * sln2;
 int lns[MAX_INIT_LABEL];

 lun = strlen(buf);
 if(buf == NULL || lun == 0)
 {
  g_print("\nERRORE: buffer vuoto o nullo");
  return -1;
 }

 while(i < lun && n < MAX_INIT_LABEL)
 {
  /* Trova la fine della riga */
  i = start;
  sln[n] = &buf[start];
  while(buf[i] != '\n' && i < lun)
   i++;

  /* Elimina i commenti */
  com = 0;
  for(j = start; j < i; j++)
   if(buf[j] == '#' || com)
   {
    buf[j] = ' ';
    com = 1;
   }

  /* Aggiorna l'indice e il contatore */
  lns[n] = i-start;
  max = (max < lns[n]) ? lns[n] : max;
  start = i+1;
  n++;
 }

 sln1 = (char *)malloc(max);
 sln2 = (char *)malloc(max);

 for(i = 0; i < n; i++)
 {
  com = 0;
  for(k = lns[i]; k >= 0; k--)
   if(sln[i][k] == '=')
    com = 1;

  if(com)
  {
   sscanf(sln[i], "%s = %s", sln1, sln2);
   g_print("\n%-2i: sln1=\"%s\"\t\t\tsln2=\"%s\"", i, sln1, sln2);
   set_init_value(sln1, sln2);
  }
  else
   g_print("\nUguale not found");
 }

 M_FREE(sln1);
 M_FREE(sln2);

 return 0;
}
