#ifdef HAVE_CONFIG_H
 # include <config.h>
#endif

#include <stdlib.h>

#ifndef SUB_CALLING
 #include <stdio.h>
 #include <gtk/gtk.h>
 #include <string.h>

 #include "callbacks.h"
 #include "interface.h"
 #include "support.h"
 #include "pfunc.h"
 #include "sfunc.h"
 #include "mfunc.h"
#endif

#include "stf-export.h"

#ifndef ANJUTA_MAKEFILE
 #include "stf-export.c"
#endif


void
chiudi_con_modifiche (GtkWidget * padre, int stato)
{
  GtkRequisition wb;

  g_print ("\nTentativo di uscire con modifiche non salvate (%d)", stato);

  if (stato & AVT_DESTROY)
    Status[S_AVVRTM] = stato;
  else
  {
    Status[S_AVVRTM] = stato;
    gtk_widget_size_request (padre, &wb);
    gtk_widget_set_sensitive (padre, FALSE);
    g_print ("\nStato di destroy non settato.");
  }
  Avvertimento = create_avvertimento ();
  MOSTRA_ON_MOUSE (Avvertimento);
}


int
get_whats_key (GdkEventKey * event)
{
  /* su archittetture PC/IBM la posizione e` 20 */
  int evt, key = (int) ((char *) event)[20];

  /* ma su architteture SPARC/SUN laposizione e` 23 */
  if (!key)
    key = (int) ((char *) event)[23];

  /* Meglio prendere il valore con un puntatore */
  evt = event->keyval;

  g_print ("\nkey: 0x%x, keyval: 0x%x = (0x%x, 0x%x)", key, evt, evt & 0xff00, evt & 0x0ff);

  return evt & 0x0ff;
}


void
g_set_window_title (GtkWidget * wind, const char *text)
{
  gchar *title;

  title = g_strdup_printf ("%s", text);
  gtk_window_set_title (GTK_WINDOW (wind), title);
  g_free (title);
}

void
g_set_statusbar (GtkWidget * wind, const char *text)
{
  GtkWidget *statusbar;

  statusbar = lookup_widget (wind, "statusbar");
  gtk_statusbar_pop (GTK_STATUSBAR (statusbar), 1);
  gtk_statusbar_push (GTK_STATUSBAR (statusbar), 1, text);
}


void
carica_rubrica (void)
{
  int k;
  long i;
  struct card *rc;

  g_print ("\nLoading rubrica file");

  Primo = NULL;			/* Inizializzo il puntatore */
  Primo = (struct card *) malloc (sizeof (struct card));	/* Alloco una struttura */
  if (Primo == NULL)
    g_print ("\nERROR: card \"Primo\" not allocated\n");

  k = 0;
  rc = Primo;			/* Inizializzazione dei puntatori */
  rc->nxt = NULL;
  for (i = 0; i < FileBufferLun; i++)
  {
    if (FileBuffer[i] == '\t')	/* Converte i tabulatori in fine di riga */
      FileBuffer[i] = 0;
    else if (FileBuffer[i] == '#')	/* Concatena tutti i nominativi */
    {
      /* Impedisce che #seq: in fondo al file sia scambiato per un nominativo */
      if (!strncasecmp (&FileBuffer[i], "#seq:", 5))
      {
	Sequenza = duplica_sequenza (&FileBuffer[i]);
	// g_print("\nRaggiunta la fine della rubrica.\nTi mostro la sequenza:\n%s", Sequenza);

	/* L'inizializzazione di S_CAMPI e` necessari per get_formato_card() */
	Status[S_CAMPI] = get_intestazione ();

	/* La lunghezza dell'intestazione e` indispensabile per stampare una card */
	LungIntst = get_formato_card (Sequenza);
      }
      else
      {
	rc->cod = &(FileBuffer[i]);	/* Iniziallizza la struttura */
	rc->nxt = (struct card *) malloc (sizeof (struct card));	/* Crea la successiva */
	if (rc->nxt == NULL)
	  g_print ("\nERROR: card n. %d not allocated\n", k);
	else
	  g_print ("\ncard n. %d allocated\n", k);

	Lista[k] = rc;		/* Inizializzazione della Lista */

	rc = rc->nxt;		/* Aggancia la prossima struttura */
	rc->nxt = NULL;		/* Inizializza il puntatore alla prossima */
	k++;
      }
    }
  }

  /* Bisogna pulire la card prima di stampare
   * la nuova altrimenti finisco per appenderla
   * se lo stato di testo risultasse non settato */
  set_testo ("", Base, "text3", TXT_CLEAN);

  /* Il numero di cards e` necessario per 
   * impostare correttamente la nuova lista */
  Status[S_CARDS] = k;
  g_print (" (%ld bytes : %d cards) \n", i, Status[S_CARDS]);
  imposta_rubrica ();

  /* Lo stato lo setto alla fine altrimenti 
   * mi prdo il ricordo di quello precedente */
  Status[S_RUBRICA] = 1;
  Status[S_SEQUENZA] = 1;
  Status[S_LISTA] = 1;
  set_sensitive_tools (IF_NEED);
}


void
imposta_rubrica (void)
{
  int i, k;
  char *str;

  g_print ("\nImposta rubrica started");

  for (k = 0; k < Status[S_CARDS]; k++)
  {
    /* RAF 2001-06-02: estensione della lettura del file
     * ad uncodice lungo un numero arbitrario di caratteri 
     */

    str = Lista[k]->cod + 1;	                    /* Il codice compare nella forma #nnnn */
	  i = 1; while(str[i] && str[i] != '"') i++;
    str[i] = 0;                                   /* deve essere convertito in un numero */
    Lista[k]->num = atoi (str);

    i++; while (Lista[k]->cod[i] != '"') i++;     /* Completa i campi piu` significativi */
		Lista[k]->cog = Lista[k]->cod + i + 1;
    i = strlen (Lista[k]->cog);                   /* "#nnn" \0 "cognome" \0 "nome" \0    */
    Lista[k]->cog[i - 1] = 0;

    while (Lista[k]->cog[i] != '"')
      i++;
    Lista[k]->nom = Lista[k]->cog + i + 1;
    i = strlen (Lista[k]->nom);
    Lista[k]->nom[i - 1] = 0;
  }

  g_print (" and works fine");

  imposta_lista ();
}


void 
esporta_rubrica_in_csv(FILE *f)
{
	RAF_DEBUG_INIT{"esporta_rubrica_in_csv"};
	StfExportOptions_t *opt;
	Sheet sheet;
	//FILE *f;
	long i = 0, j = 0;
	char *p[MAX_N_CAMPI];
	
	RAF_DEBUG("S_CARDS %d", Status[S_CARDS]);
	sheet.range.start_row = 0;
	sheet.range.start_col = 0;
	sheet.range.end_row = Status[S_CARDS]+1; // Aggiungo +1 per l'intestazione dei campi 
	sheet.range.end_col = Status[S_CAMPI];
 	RAF_DEBUG("S_CAMPI %d", Status[S_CAMPI]); 

	/* RAF 2001-06-03: Scrivo l'intestazione dei campi */
	i = sheet.range.start_row;
	for(j = sheet.range.start_col; j < sheet.range.end_col; j++) {
		RAF_DEBUG("row %ld,  col %ld", i, j);
		sheet.data[i][j] = Campi[j];
  }
  
	/* RAF 2001-06-03: scrivo la tabella dei dati */
	for(++i; i < sheet.range.end_row; i++) 
	{
		RAF_DEBUG("row %ld", i);
		estrai_campi_da_card(Lista[i-1], p);
		for(j = sheet.range.start_col; j < sheet.range.end_col; j++) {
			RAF_DEBUG("row %ld,  col %ld", i, j);
			sheet.data[i][j] = p[j];
		}
	}
	RAF_DEBUG("FINE row %ld,  col %ld", i, j);
	
	opt = stf_export_options_new();
	stf_export_options_set_default(opt);
	/*
	stf_export_options_set_terminator_type(opt, TERMINATOR_TYPE_RETURN_LINEFEED);
	stf_export_options_set_cell_separator(opt, '\t');
	stf_export_options_set_quoting_mode(opt, QUOTING_MODE_ALWAYS);
	stf_export_options_set_quoting_char(opt, '"');
	*/
	stf_export_options_sheet_list_add(opt, &sheet);	

	/* RAF 2001-06-03: per qualche ragione alla fine della sequenza si accumula rumenta
	i = strlen(Sequenza)-1;
	RAF_DEBUG("lunghezza sequenza %ld", i);
	while(Sequenza[i] != '\"' && i > 0) {
		RAF_DEBUG("%ld:%c", i, Sequenza[i]);
		i--;
	}
	Sequenza[i+1] =	'\0';
	*/
	
	/* RAF 2001-06-03: scrivo il file con il nome scelto    */
	/* RAF 2001-06-03: TODO on_salva_activate da modificare 
	M_FREE(NomeFile);
	on_salva_con_nome_activate (NULL, NULL);
	while( Status[S_FILEDIALOG] ) {
  	while (gtk_events_pending())
    	gtk_main_iteration();
	}
	
	RAF_DEBUG("Nome del file: %s", nomefile);
	if(nomefile != NULL) {
		f = fopen(nomefile, "w"); 
  */
	if(f != NULL) {
		stf_export_options_set_write_callback(opt, (StfEWriteFunc)stf_write_func, f);
		g_print("result %d", stf_export_sheet(opt, &sheet) );
	}
}



void
imposta_lista (void)
{
  int i, lnum;
  char *strn;
  GtkWidget *list, *list_item;
  GList *dlist;

  g_print ("\nImposta lista started %d", Status[S_SELISTA]);

  lnum = Lista[Status[S_SELISTA]]->num;
  ordina_lista ();
  Status[S_SELISTA] = get_selection_by_code (lnum);
  g_print (" -> %d", Status[S_SELISTA]);

  dlist = NULL;
  if (abs (Status[S_ORDINE]) == ORD_COG)
    for (i = 0; i < Status[S_CARDS]; i++)	/* Scrittura degli elementi della lista */
    {
      strn = malloc (strlen (Lista[i]->cog) + strlen (Lista[i]->nom) + 4);
      sprintf (strn, "%s %s", Lista[i]->cog, Lista[i]->nom);
      list_item = gtk_list_item_new_with_label (strn);
      dlist = g_list_prepend (dlist, list_item);
      gtk_widget_show (list_item);
      gtk_object_set_data (GTK_OBJECT (list_item), list_item_data_key,
			   Lista[i]->cod);
    }
  else
    for (i = 0; i < Status[S_CARDS]; i++)	/* Scrittura degli elementi della lista */
    {
      strn = malloc (strlen (Lista[i]->cog) + strlen (Lista[i]->nom) + 4);
      sprintf (strn, "%s %s", Lista[i]->nom, Lista[i]->cog);
      list_item = gtk_list_item_new_with_label (strn);
      dlist = g_list_prepend (dlist, list_item);
      gtk_widget_show (list_item);
      gtk_object_set_data (GTK_OBJECT (list_item), list_item_data_key,
			   Lista[i]->cod);
    }
  dlist = g_list_reverse (dlist);

  /* Visualizzazione del contenuto della lista 
   * Se la lista era gia` formata occurre pulirla  
   * ricostruire la giusta selezione */
  list = lookup_widget (GTK_WIDGET (Base), "lista");
  if (Status[S_LISTA])
  {
    Status[S_INIBIT] = S_LISTA;
    gtk_list_clear_items (GTK_LIST (list), 0, Status[S_CARDS]);
    // old = Status[S_SELISTA];
    gtk_list_prepend_items (GTK_LIST (list), dlist);
    Status[S_INIBIT] = -1;
    // Status[S_SELISTA] = old;
    g_print ("\nPippone");
    //gtk_list_select_child (GTK_LIST(list), list_item); 
    set_lista_item (Status[S_SELISTA], !S_INIBIT);
    //gtk_widget_grab_focus(list);
  }
  else
    gtk_list_prepend_items (GTK_LIST (list), dlist);


  /*set_sensitive_tools(IF_NEED); */
  g_print ("\nImposta lista returned");
}


void
ordina_lista (void)
{
  int n, ris, cont, k, ord;
  struct card *rc;

  ord = ORD_VTEST (Status[S_ORDINE]);
  g_print ("\nOrdina lista started, verso %d", ord);

  k = 0;
  cont = 1;

  if (abs (Status[S_ORDINE]) == ORD_COG)
    while (cont)
    {
      k++;
      cont = 0;
      for (n = 0; n < Status[S_CARDS] - 1; n++)
      {
	ris = strcmp (Lista[n]->cog, Lista[n + 1]->cog);
	if (ord * ris < 0)
	{
	  rc = Lista[n];
	  Lista[n] = Lista[n + 1];
	  Lista[n + 1] = rc;
	  cont = 1;
	}
	else if (!ris)
	{
	  ris = strcmp (Lista[n]->nom, Lista[n + 1]->nom);
	  if (ord * ris < 0)
	  {
	    rc = Lista[n];
	    Lista[n] = Lista[n + 1];
	    Lista[n + 1] = rc;
	    cont = 1;
	  }
	}
      }
    }
  else
    while (cont)
    {
      k++;
      cont = 0;
      for (n = 0; n < Status[S_CARDS] - 1; n++)
      {
	ris = strcmp (Lista[n]->nom, Lista[n + 1]->nom);
	if (ord * ris < 0)
	{
	  rc = Lista[n];
	  Lista[n] = Lista[n + 1];
	  Lista[n + 1] = rc;
	  cont = 1;
	}
	else if (!ris)
	{
	  ris = strcmp (Lista[n]->cog, Lista[n + 1]->cog);
	  if (ord * ris < 0)
	  {
	    rc = Lista[n];
	    Lista[n] = Lista[n + 1];
	    Lista[n + 1] = rc;
	    cont = 1;
	  }
	}
      }
    }

  Primo = Lista[0];
  for (n = 0; n < Status[S_CARDS] - 1; n++)
    Lista[n]->nxt = Lista[n + 1];
  Lista[n]->nxt = NULL;

  g_print ("\nfinito ordinamento (%d)", n);
}


void
set_ricerca (const char *strn)
{
  GtkWidget *entry;

  g_print ("\nSet ricerca started ");

  entry = lookup_widget (GTK_WIDGET (Base), "ricerca");
  gtk_entry_set_text (GTK_ENTRY (entry), strn);
  if (!strcmp (strn, ""))
    Status[S_RICERCA] = 0;
  else
    Status[S_RICERCA] = 1;
}


int
trova_ricerca (const char *strn)
{
  int i, k;

  g_print ("\nTrova ricerca started ");

  k = -1;
  if (abs (Status[S_ORDINE]) == ORD_COG)
  {
    for (i = Status[S_CARDS] - 1; i >= 0; i--)
      if (!strncasecmp (strn, Lista[i]->cog, strlen (strn)))	/* dovrebbe trovare il primo nome ma poiche` la */
	k = i;			/* lista e` organizzata in ordine inverso... */
  }
  else
  {
    for (i = Status[S_CARDS] - 1; i >= 0; i--)
      if (!strncasecmp (strn, Lista[i]->nom, strlen (strn)))	/* dovrebbe trovare il primo nome ma poiche` la */
	k = i;			/* lista e` organizzata in ordine inverso... */
  }

  return k;
}


void
reset_lista (void)
{
  GtkWidget *list;

  g_print ("\nReset lista started ");

  list = lookup_widget (GTK_WIDGET (Base), "lista");

  /* gtk_clist_clear( (GtkCList *) GTK_LIST(list)->selection); 
   * gtk_clist_remove( (GtkCList *) GTK_LIST(list)->selection, 0); */
}


void
set_lista_item (int k, int inibit)
{
  int old;
  GtkWidget *list;
  GtkList *glst;

  if (k >= 0)
  {
    g_print ("\nSet lista item started");	// OGGI
    list = lookup_widget (GTK_WIDGET (Base), "lista");
    glst = GTK_LIST (list);


    if (inibit == S_INIBIT)
    {
      g_print (" with inibition");	// OGGI
      old = Status[S_LISTA];
      Status[S_LISTA] = -1;
      gtk_list_select_item (glst, k);
      Status[S_LISTA] = old;
    }
    else
      gtk_list_select_item (glst, k);

    //gtk_widget_grab_focus(list);

    Status[S_SELISTA] = k;
    g_print ("\nSet lista item n.%d", k);
    //gtk_list_start_selection(glst);
    //gtk_list_toggle_focus_row(glst);

    //
    //gtk_window_set_focus (list->window, list);
    //gtk_widget_grab_focus(list);  

    //gtk_list_toggle_focus_row(glst);                            
    set_scrolled_window (GTK_WIDGET (Base), "scrolledwindow4", "lista",
			 ON_PAGE);

    /* Status[S_SELCODE] = Lista[Status[S_CARDS] - k - 1]->num; */
  }
  else
    g_print ("\nERRORE: codice passato negativo");
}


int
get_selection_by_code (int code)
{
  int i, n = -1;

  for (i = 0; i < Status[S_CARDS]; i++)
    if (Lista[i]->num == code)
      n = i;

  return n;
}


void
free_memory (void)
{
  int k;
  long i;
  struct stringa *crn, *crf;
  GtkWidget *list;

  g_print ("\nFree memory starting");

  /* ATTENZIONE: free_memory() non e` adatto ad
   * essere chiamato all'uscita del programma */


  /* Questo lo facciamo quando la rubrica e` caricata`
   * perche` altrimenti va tutto a scatafascio.
   * if ( Status[S_TESTO] )
   * set_testo("", Base, "text3", TXT_CLEAN);
   * Status[S_TESTO] = 0;
   * 
   * if ( Status[S_RICERCA] )
   * set_ricerca("");
   * Status[S_RICERCA] = 0;
   */
  Status[S_TESTO] = 0;
  Status[S_RICERCA] = 0;


  /* Prima di allocare altro spazio svuoto il precedente
   * Non posso deallocare NomeFile e SoloNomeFile perche`
   * contengono informazioni e sono stati reallocati di gia` */
  if (Status[S_BUFFER])
  {
    g_print (" #1");
    if (Sequenza != NULL)
      M_FREE (Sequenza);
    if (FileBuffer != NULL)
      M_FREE (FileBuffer);
    Status[S_BUFFER] = 0;
  }

  if (Status[S_LISTA])		/* Se la lista era gia` formata occurre pulirla */
  {
    g_print (" #2");
    list = lookup_widget (GTK_WIDGET (Base), "lista");
    gtk_list_clear_items (GTK_LIST (list), 0, Status[S_CARDS]);
    Lista[S_LISTA] = 0;
  }

  if (Status[S_CARDS])		/* Occore liberare tutte le strutture */
  {
    g_print (" #3");
    for (i = 0; i < Status[S_CARDS]; i++)
      M_FREE (Lista[i]);
    Status[S_CARDS] = 0;
  }

  k = 0;
  if (Status[S_CAMPI])
  {
    g_print (" #4");

    crf = Carta.next;
    if (Carta.text != NULL)
      M_FREE (Carta.text);
    while (crf != NULL && crf->next != NULL)
    {
      crn = crf->next;
      if (crf != NULL)
	M_FREE (crf->text);
      M_FREE (crf);
      crf = crn->next;
      k++;
    }

    for (i = 0; i < Status[S_CAMPI]; i++)
      if (Campi[i] != NULL)
	M_FREE (Campi[i]);
    for (i = 0; i < Status[S_CAMPI]; i++)
      if (ContenutoCampi[i] != NULL)
	M_FREE (ContenutoCampi[i]);

    Status[S_CAMPI] = 0;
  }

  g_print (" (%d)", k);
  for (i = 0; i < NUM_STATUS_FLAGS; i++)	/* Inizializzo anche lo status che male non fa */
    if (i != S_ORDINE)		/* l'ordinamento  pi comodo mantenerlo */
      Status[i] = 0;

  return;
}


void
set_scrolled_window (GtkWidget * container, const char *scrolledwindow,
		     const char *lista, int mth)
{
  GtkAdjustment *Adj;
  GtkObject *list_item;
  GtkWidget *scrollbar;
  gint scroll_y;
  int y1, y2, y;

  list_item = get_selected_list_item (container, lista);
  if (list_item == NULL)
  {
    g_print
      ("\nERRORE: get_selected_list_item ha ritornato un puntatore nullo\n        controllare Status[S_LISTA] = %d deve essere 1",
       Status[S_LISTA]);
    return;
  }
  scroll_y = GTK_WIDGET (list_item)->allocation.y;

  scrollbar = lookup_widget (container, scrolledwindow);
  if (scrollbar == NULL)
  {
    g_print
      ("\nERRORE: la scrollbar ha ritornato un puntatore nullo\n        controllare Status[S_LISTA] = %d deve essere 1",
       Status[S_LISTA]);
    return;
  }

  Adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scrollbar));
  g_print ("\nset_scrolled_window started on %s,%s \n( %d < %d + %d < %d )",
	   scrolledwindow, lista, (int) Adj->value, (int) scroll_y,
	   (int) Adj->step_increment, (int) (Adj->value + Adj->page_size));

  /* POTENZIALE PROBLEMA */
  /* Accade che lo scroll correto della lista venga fatto da imposta_listacampi
   * in seguito la lista si trova in una condizione di scroll_y = -1  */
  if (scroll_y < 0)
    return;

  if (mth == ON_FIRST)
  {
    g_print ("\nCalling vscroll in ON_FIRST mode");
    set_vscroll_window (container, scrolledwindow, scroll_y);
  }
  else if (mth == ON_PAGE)
  {
    y1 = (int) (Adj->value);
    y2 = (int) (Adj->value + Adj->page_size - Adj->step_increment);

    /* Controllo se scroll_y  anche soltanto parzialmente fuori
     * dalla finestra di visualizzazione */
    if (y1 < scroll_y && scroll_y < y2)
      g_print ("\nNo need call to vscroll");
    else
    {
      y1 = (int) (scroll_y / Adj->page_size);
      y2 = (int) ((scroll_y + Adj->step_increment) / Adj->page_size);
      g_print ("\nCalling vscroll in ON_PAGE mode: y1=%d, y2=%d", y1, y2);

      /* se sono diversi significa che sta a cavallo di due pagine */
      if (y1 == y2)
	y = y1 * Adj->page_size;
      else
	y = scroll_y;

      set_vscroll_window (container, scrolledwindow, y);
    }
  }
  else
    g_print ("\nNo call to vscroll because different mode");


  return;
}


void
set_vscroll_window (GtkWidget * container, const char *scrolledwindow,
		    int start)
{
  GtkAdjustment *Adj;
  GtkWidget *scrollbar;

  scrollbar = lookup_widget (container, scrolledwindow);

  /* g_print("\nSet scrolledwindow started (%d): %s %s", scroll_y, scrolledwindow, lista); */
  /* GTK_WIDGET(scrollbar)->allocation.y = scroll_y+14;
   * GTK_WIDGET (list_item)->allocation.y = scroll_y+14;
   * GTK_WIDGET (lst)->allocation.y = scroll_y+14; 
   */

  Adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scrollbar));
  g_print
    ("\n\nValori: v(n-1)=%3.1f, v(n)=%3.1f, l=%3.1f, u=%3.1f, s=%3.1f, p=%3.1f, s=%3.1f\n",
     Adj->value, (float) start, Adj->lower, Adj->upper, Adj->step_increment,
     Adj->page_increment, Adj->page_size);

  if (start != Adj->value) ;
  {
    if (start >= 0)
    {
      if (start + Adj->page_size > Adj->upper)
	start = Adj->upper - Adj->page_size;
      Adj->value = start;
      gtk_scrolled_window_set_vadjustment (GTK_SCROLLED_WINDOW (scrollbar),
					   Adj);
      gtk_adjustment_value_changed (Adj);
    }
    else
      g_print ("\nERROR: start must be positve integer (%d)", start);
  }

  return;
}


GtkObject *
get_selected_list_item (GtkWidget * container, const char *widget)
{
  GtkWidget *lst;
  GList *dlist;
  GtkObject *list_item;

  g_print ("\nGet selected list item started: %s", widget);

  list_item = NULL;
  if (Status[S_LISTA] > 0)
  {
    lst = lookup_widget (container, widget);
    dlist = GTK_LIST (lst)->selection;
    // OGGI list_item = GTK_OBJECT(dlist->data);
    // OGGI
    while (dlist)		// OGGI
    {				// OGGI
      list_item = GTK_OBJECT (dlist->data);	// OGGI
      dlist = dlist->next;	// OGGI
    }				// OGGI
  }

  return list_item;
}


void
modifica_ripristina (void)
{
  GtkWidget *text;

  Modifica = create_modifica ();
  text = lookup_widget (GTK_WIDGET (Modifica), "card_preview");
  gtk_text_set_word_wrap (GTK_TEXT (text), 1);

  imposta_listacampi (Modifica, "listacampi");
  /* RO16 stampa_card(CardCopia, Modifica, "card_preview"); */
  set_vscroll_window (Modifica, "scrolledwindow7", 0);
}


void
set_sensitive_tools (int force)
{
  static GtkWidget *menu[20];
  static GtkWidget *oldBase = NULL;
  static int OSRubrica = -99, i;
  static int OSModifica = -99;

  /* I puntatori devono essere inizializzati oppure aggiornati
   * se la finestra Base viene distrutta e poi ricreata */
  if (oldBase != Base || force)
  {
    for (i = 0; i < 20; i++)
      menu[i] = NULL;

    menu[0] = lookup_widget (Base, "salva_con_nome");
    menu[1] = lookup_widget (Base, "cerca");
    menu[2] = lookup_widget (Base, "aggiungi");
    menu[3] = lookup_widget (Base, "modifica");
    menu[4] = lookup_widget (Base, "button_rinomina");
    menu[5] = lookup_widget (Base, "button_aggiungi");
    menu[6] = lookup_widget (Base, "button_modifica");
    menu[7] = lookup_widget (Base, "esporta");
    menu[8] = lookup_widget (Base, "stampa");

    menu[10] = lookup_widget (Base, "salva");
    menu[11] = lookup_widget (Base, "button_salva");

    g_print ("\nBase pointer changed");
  }

  /* Le sensitive dei tools cambiano solo se c'e` stata una
   * modifca nello Status[RUBRICA] oppure la prima volta */
  if (OSRubrica != Status[S_RUBRICA] || force)
  {
    OSRubrica = Status[S_RUBRICA];
    for (i = 0; i < 10 && menu[i]; i++)
      gtk_widget_set_sensitive (menu[i], OSRubrica);
  }

  /* Quelli di indice superiore al nove vengono attivati
   * solo quando  stata apportata una modifica alla rubrica */
  if (OSModifica != Status[S_MODIFICA] || force)
  {
    OSModifica = Status[S_MODIFICA];
    for (i = 10; i < 20 && menu[i]; i++)
      gtk_widget_set_sensitive (menu[i], OSModifica);
  }
}

void
aggiorna_sequenza_modellata (const char *newseq)
{
  GtkWidget *prvtxt;

  /* Se la nuova sequenza e` uguale alla prima non si deve
   * deallocare la vecchia vuol dire che occorre solo aggiornare */
  if (newseq != Sequenza)
  {
    M_FREE (Sequenza);
    Sequenza = (char *) newseq;
  }

  // g_print("(lun=%ld/%d)\n%s\n", lun, strlen(Sequenza), Sequenza);

  LungIntst = get_formato_card (Sequenza);
  if (LungIntst < 0)
    return;

  if (Status[S_SELISTA] < 0)
    stampa_card (Primo, Creazione, "crea_preview");
  else
    stampa_card (Lista[Status[S_SELISTA]], Creazione, "crea_preview");
  set_vscroll_window (Creazione, "scrolledwindow9", 0);
  prvtxt = lookup_widget (Creazione, "crea_preview");
  gtk_text_set_editable (GTK_TEXT (prvtxt), 0);

  /* Non e` detto che venga modificato ogni volta */
  /*Status[S_SEQUENZA] = -1; */
}
